@@ -353,7 +353,7 @@ print.simple_di_det_ipm <- function(x,
     msg    <- c(msg, l_msg)
 
     if(check_conv &&
-       !is_conv_to_asymptotic(x)) {
+       !all(is_conv_to_asymptotic(x))) {
 
       # Captures the name of the model that the user gave rather than
       # just print "x isn't converged"
@@ -555,7 +555,7 @@ print.general_di_det_ipm <- function(x,
 
   }
 
-  if(check_conv && ! is_conv_to_asymptotic(x)) {
+  if(check_conv && !all(is_conv_to_asymptotic(x))) {
 
     message(
       paste(mod_nm,
@@ -1253,15 +1253,19 @@ lambda.general_di_stoch_param_ipm <- function(ipm,
 #' @rdname lambda
 #' @export
 
-lambda.simple_dd_det_ipm <- function(ipm, type_lambda = "all", ..., log = FALSE) {
+lambda.simple_dd_det_ipm <- function(ipm,
+                                     type_lambda = "all",
+                                     ...,
+                                     log = FALSE) {
 
   .check_lambda_args(ipm, type_lambda)
 
-  all_lams <- switch(type_lambda,
-                     'all'  = TRUE,
-                     'last' = FALSE,
-                     'stochastic' = stop("Cannot compute stochastic lambda for deterministic IPM",
-                                         call. = FALSE))
+  all_lams <- switch(
+    type_lambda,
+    'all'  = TRUE,
+    'last' = FALSE,
+    'stochastic' = stop("Cannot compute stochastic lambda for deterministic IPM",
+                        call. = FALSE))
 
   out <- .lambda_pop_size(ipm, all_lambdas = all_lams)
 
@@ -1298,6 +1302,7 @@ lambda.simple_dd_stoch_kern_ipm <- function(ipm,
                 'stochastic' = .thin_stoch_lambda(temp, burn_ind, log))
 
   if(type_lambda == "stochastic") {
+
     if(log) {
       message("log(lambda) is returned by default for stochastic models. Set ",
               "'log = FALSE' for lambda on linear scale.")
@@ -1851,10 +1856,6 @@ right_ev.simple_di_det_ipm <- function(ipm,
 
   mod_nm <- deparse(substitute(ipm))
 
-  # Identify state variable name
-
-  pop_nm <- .get_pop_nm_simple(ipm)
-
   # if it's already been iterated to convergence, we don't have much work to do.
 
   if(.already_iterated(ipm)) {
@@ -1862,11 +1863,10 @@ right_ev.simple_di_det_ipm <- function(ipm,
     # get index for population vector of final iteration
     final_it <- dim(ipm$pop_state[[1]])[2]
 
-    if(is_conv_to_asymptotic(ipm,
-                             tol = tolerance)) {
+    if(all(is_conv_to_asymptotic(ipm,
+                                 tolerance = tolerance))) {
 
-      out    <- ipm$pop_state[[1]][ , final_it]
-      out_nm <- paste(pop_nm, 'w', sep = "_")
+      out    <- .extract_conv_ev_simple(ipm$pop_state)
 
     } else {
 
@@ -1889,29 +1889,29 @@ right_ev.simple_di_det_ipm <- function(ipm,
         )
       )
 
-      init_pop_vec <- ipm$pop_state[[1]][ , final_it]
-
-      pop_nm       <- ipm$proto_ipm$state_var %>%
-        unlist() %>%
-        unique() %>%
-        .[1] %>%
-        paste('n_', ., sep = "")
-
-      pop_nm <- rlang::ensym(pop_nm)
+      init_pop_vec <- lapply(ipm$pop_state[!grepl("lambda", names(ipm$pop_state))],
+                             function(x, final_it) {
+                               x[ , final_it]
+                             },
+                             final_it = ncol(ipm$pop_state[[1]]))
+#
+#       pop_nm       <- ipm$proto_ipm$state_var %>%
+#         unlist() %>%
+#         unique() %>%
+#         .[1] %>%
+#         paste('n_', ., sep = "")
+#
+#       pop_nm <- rlang::ensym(pop_nm)
 
       test_conv <- ipm$proto_ipm %>%
-        define_pop_state(!! pop_nm := init_pop_vec) %>%
+        define_pop_state(!!! init_pop_vec) %>%
         make_ipm(iterate    = TRUE,
                  iterations = iterations)
 
-      if(is_conv_to_asymptotic(test_conv,
-                               tol = tolerance)) {
+      if(all(is_conv_to_asymptotic(test_conv,
+                                   tolerance = tolerance))) {
 
-        final_it <- dim(test_conv$pop_state[[1]])[2]
-
-        out      <- test_conv$pop_state[[1]][ , final_it]
-
-        out_nm   <- paste(pop_nm, 'w', sep = "_")
+        out    <- .extract_conv_ev_simple(test_conv$pop_state)
 
         message('model is now converged :)')
 
@@ -1951,10 +1951,11 @@ right_ev.simple_di_det_ipm <- function(ipm,
     # A model that hasn't been iterated yet
 
     # Create variables for internal usage
+    pop_nm <- .get_pop_nm_simple(ipm)
 
-    pop_states  <- paste('n', pop_nm, c('t', 't_1'), sep = '_')
+    pop_states  <- paste('n', pop_nm, sep = '_')
 
-       # Final step is to generate an initial pop_state. This is always just
+    # Final step is to generate an initial pop_state. This is always just
     # vector drawn from a random uniform distribution
 
     len_pop_state <- dim(ipm$sub_kernels[[1]])[1]
@@ -1963,19 +1964,16 @@ right_ev.simple_di_det_ipm <- function(ipm,
     # Drop _t for defining the initial population vector so define_pop_state
     # doesn't complain
 
-    pop_states[1] <- gsub("_t$", "", pop_states[1])
-
     test_conv     <- ipm$proto_ipm %>%
       define_pop_state(!! pop_states[1] := init_pop) %>%
       make_ipm(iterate = TRUE,
                iterations = iterations,
                normalize_pop_size = TRUE)
 
-    if(is_conv_to_asymptotic(test_conv,
-                             tol = tolerance)) {
+    if(all(is_conv_to_asymptotic(test_conv,
+                                 tolerance = tolerance))) {
 
-      out    <- test_conv$pop_state[[1]][ , (iterations + 1)]
-      out_nm <- paste(pop_nm, 'w', sep = "_")
+      out    <- .extract_conv_ev_simple(test_conv$pop_state)
 
     } else {
 
@@ -1998,8 +1996,13 @@ right_ev.simple_di_det_ipm <- function(ipm,
 
   # Stuff into a list and standardize
 
-  out <- rlang::list2(!! out_nm := (out / sum(out)))
+  # out <- rlang::list2(!! out_nm := (out / sum(out)))
 
+  out <- Filter(function(x) !any(is.na(x)), out)
+
+  # Replaces the leading n_ with a trailing _v
+  names(out) <- substr(names(out), 3, nchar(names(out)))
+  names(out) <- paste(names(out), 'w', sep = '_')
   class(out) <- "ipmr_w"
 
   return(out)
@@ -2037,8 +2040,9 @@ right_ev.simple_di_stoch_kern_ipm <- function(ipm,
 
   out[[1]] <- out[[1]] / colSums(out[[1]])
 
-  names(out) <- paste(pop_nm, 'w', sep = "_")
-
+  # Replaces the leading n_ with a trailing _v
+  names(out) <- substr(names(out), 3, nchar(names(out)))
+  names(out) <- paste(names(out), 'w', sep = '_')
   class(out) <- "ipmr_w"
 
   return(out)
@@ -2063,10 +2067,10 @@ right_ev.general_di_det_ipm <- function(ipm,
 
   final_it  <- dim(ipm$pop_state[[1]])[2]
 
-  if(is_conv_to_asymptotic(ipm,
-                           tol = tolerance)) {
+  if(all(is_conv_to_asymptotic(ipm,
+                           tolerance = tolerance))) {
 
-    out <- .extract_conv_ev_general(ipm$pop_state)
+    out <- .extract_conv_ev_general(ipm$pop_state, ipm$proto_ipm)
 
   } else {
 
@@ -2087,12 +2091,15 @@ right_ev.general_di_det_ipm <- function(ipm,
       )
     )
 
-    use_pop_state       <- ipm$pop_state[names(ipm$pop_state) != 'lambda']
+    use_pop_state       <- ipm$pop_state[!grepl("lambda", names(ipm$pop_state))]
 
     init_pop_vec        <- lapply(use_pop_state,
                                   function(x, final_it) x[ , final_it],
                                   final_it = final_it)
 
+    # Remove NAs generated by the unsubstituted pop_state place holders
+    init_pop_vec <- Filter(f = function(y) !any(is.na(y)), x = init_pop_vec)
+
     names(init_pop_vec) <- gsub('pop_state', 'n', names(init_pop_vec))
 
     test_conv           <- ipm$proto_ipm %>%
@@ -2102,10 +2109,10 @@ right_ev.general_di_det_ipm <- function(ipm,
       make_ipm(iterate    = TRUE,
                iterations = iterations)
 
-    if(is_conv_to_asymptotic(test_conv,
-                             tol = tolerance)) {
+    if(all(is_conv_to_asymptotic(test_conv,
+                                 tolerance = tolerance))) {
 
-      out <- .extract_conv_ev_general(test_conv$pop_state)
+      out <- .extract_conv_ev_general(test_conv$pop_state, test_conv$proto_ipm)
 
     } else {
 
@@ -2126,6 +2133,8 @@ right_ev.general_di_det_ipm <- function(ipm,
     }
   }
 
+  out <- Filter(function(x) !any(is.na(x)), out)
+
   # Replaces the leading n_ with a trailing _v
   names(out) <- substr(names(out), 3, nchar(names(out)))
   names(out) <- paste(names(out), 'w', sep = '_')
@@ -2169,6 +2178,8 @@ right_ev.general_di_stoch_kern_ipm <- function(ipm,
     }
   }
 
+  out <- Filter(function(x) !any(is.na(x)), out)
+
   # Replaces the leading n_ with a trailing _v
   names(out) <- substr(names(out), 3, nchar(names(out)))
   names(out) <- paste(names(out), 'w', sep = '_')
@@ -2209,9 +2220,21 @@ left_ev.simple_di_det_ipm <- function(ipm,
 
   pop_nm <- .get_pop_nm_simple(ipm)
 
+  if(any(ipm$proto_ipm$uses_par_sets)) {
+
+    ps_nms <- .flatten_to_depth(ipm$proto_ipm$par_set_indices, 1L) %>%
+      names() %>%
+      unique()
+    ps_nms <- ps_nms[ps_nms != "levels"]
+
+    ps_suff <- paste(ps_nms, collapse = "_")
+
+    pop_nm <- paste(pop_nm, ps_suff, sep = "_")
+  }
+
   # Create variables for internal usage
 
-  pop_states  <- paste('n', pop_nm, c('t', 't_1'), sep = '_')
+  pop_states  <- paste('n', pop_nm, sep = '_')
 
   # Final step is to generate an initial pop_state. This is always just
   # vector drawn from a random uniform distribution
@@ -2219,11 +2242,6 @@ left_ev.simple_di_det_ipm <- function(ipm,
   len_pop_state <- dim(ipm$sub_kernels[[1]])[1]
   init_pop      <- stats::runif(len_pop_state)
 
-  # Drop _t for defining the initial population vector so define_pop_state
-  # doesn't complain
-
-  pop_states[1] <- gsub("_t$", "", pop_states[1])
-
   test_conv     <- ipm$proto_ipm %>%
     define_pop_state(!! pop_states[1] := init_pop) %>%
     make_ipm(iterate = TRUE,
@@ -2231,11 +2249,11 @@ left_ev.simple_di_det_ipm <- function(ipm,
              iteration_direction = "left",
              normalize_pop_size = TRUE)
 
-  if(is_conv_to_asymptotic(test_conv,
-                           tol = tolerance)) {
+  if(all(is_conv_to_asymptotic(test_conv,
+                               tolerance = tolerance))) {
 
-    out    <- test_conv$pop_state[[1]][ , (iterations + 1)]
-    out_nm <- paste(pop_nm, 'v', sep = "_")
+    out    <- .extract_conv_ev_simple(test_conv$pop_state)
+    # out_nm <- paste(pop_nm, 'v', sep = "_")
 
   } else {
 
@@ -2254,9 +2272,11 @@ left_ev.simple_di_det_ipm <- function(ipm,
     return(NA_real_)
   }
 
-# Stuff into a list and standardize
+  # Stuff into a list and standardize
 
-  out <- rlang::list2(!! out_nm := (out / sum(out)))
+  out        <- Filter(function(x) !any(is.na(x)), out)
+  names(out) <- substr(names(out), 3, nchar(names(out)))
+  names(out) <- paste(names(out), 'v', sep = '_')
   class(out) <- "ipmr_v"
 
   return(out)
@@ -2370,10 +2390,13 @@ left_ev.general_di_det_ipm <- function(ipm,
 
   mod_nm    <- deparse(substitute(ipm))
 
-  use_pop_state       <- ipm$pop_state[names(ipm$pop_state) != 'lambda']
+  use_pop_state       <- ipm$pop_state[!grepl("lambda", names(ipm$pop_state))]
 
   init_pop_vec       <- lapply(use_pop_state, function(x) x[ , 1])
 
+  # Remove NAs generated by the unsubstituted pop_state place holders
+  init_pop_vec <- Filter(f = function(y) !any(is.na(y)), x = init_pop_vec)
+
   names(init_pop_vec) <- gsub('pop_state', 'n', names(init_pop_vec))
 
   test_conv           <- ipm$proto_ipm %>%
@@ -2385,9 +2408,9 @@ left_ev.general_di_det_ipm <- function(ipm,
              iteration_direction = "left",
              normalize_pop_size = TRUE)
 
-  if(is_conv_to_asymptotic(test_conv, tol = tolerance)) {
+  if(all(is_conv_to_asymptotic(test_conv, tolerance = tolerance))) {
 
-    out <- .extract_conv_ev_general(test_conv$pop_state)
+    out <- .extract_conv_ev_general(test_conv$pop_state, test_conv$proto_ipm)
 
   } else {
 
@@ -2407,6 +2430,7 @@ left_ev.general_di_det_ipm <- function(ipm,
 
   }
 
+  out <- Filter(function(x) !any(is.na(x)), out)
   # Replaces the leading n_ with a trailing _v
   names(out) <- substr(names(out), 3, nchar(names(out)))
   names(out) <- paste(names(out), 'v', sep = '_')
@@ -2483,6 +2507,14 @@ left_ev.general_di_stoch_param_ipm <- function(ipm,
   sub_kernels <- ipm$sub_kernels
   proto_ipm   <- ipm$proto_ipm
 
+  usr_its <- ncol(ipm$pop_state[[1]]) - 1
+
+  if(usr_its < iterations) {
+
+    stop("'iterations' should be less than or equal to the number of iterations in 'make_ipm()'!")
+
+  }
+
   # Set up the model iteration expressions and initial population state.
   # This is a leaner version of make_ipm.general_di_stoch_param, because
   # we don't need all the user functions or other stuff - just the basic
@@ -2555,8 +2587,7 @@ left_ev.general_di_stoch_param_ipm <- function(ipm,
                 function(x, burn_seq) x[ , burn_seq, drop = FALSE],
                 burn_seq = burn_in_seq)
 
-
-  names(out) <- substr(names(out), 3, nchar(names(out)))
+  names(out) <- gsub("pop_state_", "", names(out))
   names(out) <- paste(names(out), 'v', sep = '_')
 
   class(out) <- "ipmr_v"
