@@ -43,7 +43,7 @@
 
     for(i in seq_along(levs)) {
 
-      use_lev   <- levs[i]
+      use_lev   <- paste("(_", levs[i], ")$", sep = "")
 
       use_kerns <- sub_kern_list[grepl(use_lev, names(sub_kern_list))]
 
@@ -135,7 +135,7 @@
 
     for(i in seq_len(iterations)) {
 
-      # Simple model, no par_setarchical stuff going on. We can just
+      # Simple model, no par_set stuff going on. We can just
       # stick the kernels and their expressions right into eval_general_det
 
       .bind_iter_var(main_env, i)
@@ -196,9 +196,9 @@
       rlang::env_bind(.env = main_env,
                       !!! pop_list_t_1)
 
-    } # End no par_setarchical stuff
+    } # End no par_set stuff
 
-  } # End if(par_setarchical){} else {}
+  } # End if(par_set){} else {}
 
   # Remove the NA at 1 - it is a dummy so that purrr::map2 can work
   # properly
@@ -483,7 +483,7 @@
 
     for(i in seq_along(levs)) {
 
-      use_lev <- levs[i]
+      use_lev <- paste("(_", levs[i], ")$", sep = "")
 
       use_kerns <- sub_kern_list[grepl(use_lev, names(sub_kern_list))]
 
@@ -936,7 +936,7 @@
 
     for(j in seq_along(levs)) {
 
-      use_lev   <- levs[j]
+      use_lev   <- paste("(_", levs[j], ")$", sep = "")
 
       use_kerns <- sub_kern_list[grepl(use_lev, names(sub_kern_list))]
 
@@ -1369,7 +1369,7 @@
 
     for(j in seq_along(levs)) {
 
-      use_lev   <- levs[j]
+      use_lev   <- paste("(_", levs[j], ")$", sep = "")
 
       use_kerns <- sub_kern_list[grepl(use_lev, names(sub_kern_list))]
 
